/*
*  DynViscosityUnit - Base class for all dynamic viscosity unit objects.
*
*  Copyright (C) 1998 - 2002 by Joseph A. Huwaldt <jhuwaldt@knology.net>.  
*  All rights reserved.
*
*  This library is free software; you can redistribute it and/or
*  modify it under the terms of the GNU Library General Public
*  License as published by the Free Software Foundation; either
*  version 2 of the License, or (at your option) any later version.
*
*  This library is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*  Library General Public License for more details.
**/
package jahuwaldt.tools.units;


/**
*  <p>  An abstract class that serves as the base class for various
*       dynamic viscosity units such as lbf-s/ft^2, poise, kg/m-s, etc.
*       The reference unit for this class is "Poise".   </p>
*
*  <p>  Note:  Absolute or dynamic viscosity (mu) is properly expressed in force
*       units according to it's definition.  However, in heat transfer and fluid
*       mechanics, it is usually expressed in mass-equivelant units to avoid
*       the use of a conversion factor in Reynolds number.  The proper force units
*       for dynamic viscosity are Newton-sec/m^2, but they are seldom if ever used.
*  </p>
*
*  <p>  WARNING: Be aware that when converting between mass and force units, a standard
*       terrestrial acceleration due to gravity of 9.80665 m/sec^2 is assumed here!
*  </p>
*
*  <p>  Modified by:  Joseph A. Huwaldt  </p>
*
*  @author   Joseph A. Huwaldt   Date:  January 31, 1999
*  @version  November 17, 1999
**/
public abstract class DynViscosityUnit extends Unit {

	/**
	*  Converts a value with the current units to a value with
	*  the new units specified.
	*
	*  @param  value    The value in the "currentUnits" to be converted to "newUnit".
	*  @param  newUnit  The new unit system to convert value to.
	*  @return The value converted to the new units.
	**/
	public double convert( double value, Unit newUnit ) throws CannotConvertException {
		if ( ! (newUnit instanceof DynViscosityUnit) ) {
			String oldName = this.getClass().getName();
			String newName = newUnit.getClass().getName();
			throw new CannotConvertException( oldName + " to " + newName );
		}
		value *= newUnit.factor() / this.factor();
		return value;
	}

	/**
	*  Returns the reference unit for this unit type.
	*
	*  @return  The reference unit for this type of unit object.
	**/
	public  Unit getRefUnit() {
		return Poise.getInstance();
	}


}


