/*
*  MassForceUnit - Base class for all mass/force unit objects.
*
*  Copyright (C) 1998-2002 by Joseph A. Huwaldt <jhuwaldt@knology.net>.  
*  All rights reserved.
*
*  This library is free software; you can redistribute it and/or
*  modify it under the terms of the GNU Library General Public
*  License as published by the Free Software Foundation; either
*  version 2 of the License, or (at your option) any later version.
*
*  This library is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*  Library General Public License for more details.
**/
package jahuwaldt.tools.units;



/**
*  <p>  An abstract class that serves as the base class for various
*       mass, and weight equivelent mass units such as grams, slugs & pounds.
*       This class also serves as the base class for force units such as
*       Newtons and poundals which can be converted to and from mass units
*       if you assume a standard acceleration due to gravity.
*       The reference unit for this class is "Kilograms".
*  </p>
*
*  <p>  WARNING:  Be aware that when converting between mass and force
*                 units, a standard terrestrial acceleration due to
*                 gravity of 9.80665 m/sec^2 is assumed here!
*  </p>
*
*  <p>  Modified by:  Joseph A. Huwaldt  </p>
*
*  @author   Joseph A. Huwaldt   Date:  December 18, 1997
*  @version  November 19, 1999
**/
public abstract class MassForceUnit extends Unit {

	/**
	*  <p>  Converts a value with the current units to a value with
	*       the new units specified.
	*  </p>
	*
	*  <p>  WARNING:   Be careful converting between mass and force
	*                  units!  When you do that, the assumption is made
	*                  that the conversion is done under standard
	*                  terrestrial gravitational acceleration (g).
	*  </p>
	*
	*  @param   value    The value in the "currentUnits" to be converted
	*                    to "newUnit".
	*  @param   newUnit  The new unit system to convert value to.
	*  @return  The value converted to the new units.
	**/
	public double convert (double value, Unit newUnit) throws CannotConvertException {
		if (!(newUnit instanceof MassForceUnit)) {
			String oldName = this.getClass().getName();
			String newName = newUnit.getClass().getName();
			throw new CannotConvertException(oldName + " to " + newName);
		}
		value *= newUnit.factor()/this.factor();
		return value;
	}


	/**
	*  Returns the reference unit for this unit type.
	**/
	public Unit getRefUnit() { return Kilograms.getInstance(); };

}

