/*
*  TemperatureUnit - Base class for all temperature unit objects.
*
*  Copyright (C) 1998 - 2002 by Joseph A. Huwaldt <jhuwaldt@knology.net>.  
*  All rights reserved.
*
*  This library is free software; you can redistribute it and/or
*  modify it under the terms of the GNU Library General Public
*  License as published by the Free Software Foundation; either
*  version 2 of the License, or (at your option) any later version.
*
*  This library is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*  Library General Public License for more details.
**/
package jahuwaldt.tools.units;


/**
*  <p>  An abstract class that serves as the base class for various
*       temperature units such as deg Farenheit, deg Celsius, etc.
*       The reference unit for this class is "DegKelvin".  </p>
*
*  <p>  WARNING:  Temperatures are a little different than most units in that there is
*                 not a simple factor difference, but also an offset.  The method
*                 cfactor() will only return the factor part, NOT the offset!!!  </p>
*
*  <p>  Modified by:  Joseph A. Huwaldt  </p>
*
*  @author   Joseph A. Huwaldt   Date:  December 18, 1997
*  @version  March 26, 2000
**/
abstract public class TemperatureUnit extends Unit {

	/**
	*  Converts a value with the current units to a value with
	*  the new units specified.
	*
	*  @param   value    The value in the "currentUnits" to be converted to "newUnit".
	*  @param   newUnit  The new unit system to convert value to.
	*  @return  The value converted to the new units.
	**/
	public double convert (double value, Unit newUnit) throws CannotConvertException {
		if (!(newUnit instanceof TemperatureUnit)) {
			String oldName = this.getClass().getName();
			String newName = newUnit.getClass().getName();
			throw new CannotConvertException(oldName + " to " + newName);
		}
		TemperatureUnit	newUnitT = (TemperatureUnit)newUnit;
		value = newUnitT.factor()/this.factor()*(value + this.offset()) - newUnitT.offset();
		return value;
	}


	/**
	*  <p>  Converts a value with this unit to a value with the reference units
	*       for this unit.  This is more efficient than the following:  </p>
	*
	*  <p>      valueRU = theUnits.convert(value, theUnits.getRefUnit() );  </p>
	*
	*  <p>  Note:  This version works properly with Temperature units.  </p>
	*
	*  @param   value  The value in the "this" units to be converted to "Reference Units".
	*  @return  The value converted to the reference units.
	**/
	public double convertToRef(double value) {
		//	"newUnit" factor is always 1.0 and "newUnit" offset is always 0.0.
		return (value + this.offset())/this.factor();
	}


	/**
	*  <p>  Converts a value with this unit's reference units to a value with
	*       this unit.  </p>
	*
	*  <p>  Note:  This version works properly with Temperature units.  </p>
	*
	*  @param   value  The value in the "Reference Units" to be converted to "newUnit".
	*  @return  The value converted to the new units.
	**/
	public double convertFromRef(double value) {
		//	"oldUnit" factor is always 1.0 and "oldUnit" offset is always 0.0.
		return this.factor()*value - this.offset();
	}


	/**
	*  Returns the reference unit for this unit type.
	*
	*  @return  The reference unit for this type of unit object.
	**/
	public Unit getRefUnit() { return Kelvin.getInstance(); }


	abstract protected double offset();

}

