/*
*  Unit - Base class for all unit objects.
*
*  Copyright (C) 1998 - 2002 by Joseph A. Huwaldt <jhuwaldt@knology.net>.  
*  All rights reserved.
*
*  This library is free software; you can redistribute it and/or
*  modify it under the terms of the GNU Library General Public
*  License as published by the Free Software Foundation; either
*  version 2 of the License, or (at your option) any later version.
*
*  This library is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*  Library General Public License for more details.
**/
package jahuwaldt.tools.units;


/**
*  An abstract class that serves as the base class for various
*  unit object types.  Provides some standard functionality
*  used by all unit objects.
*
*  <p>  Modified by:  Joseph A. Huwaldt  </p>
*
*  @author   Joseph A. Huwaldt   Date: December 16, 1997
*  @version  May 19, 2002
**/
public abstract class Unit implements Cloneable, java.io.Serializable {

	/**
	*  The name of this unit.
	**/
	protected String name;

	/**
	*  The abreviated name of this unit.
	**/
	protected String abv;

	/**
	*  The conversion to the reference unit is exact if true.
	**/
	protected boolean exact = false;

	//-----------------------------------------------------------------------------
	/**
	*  Converts a value with the current units to a value with
	*  the new units specified.
	*
	*  @param   value  The value in the "currentUnits" to be converted to "newUnit".
	*  @return  The value converted to the new units.
	**/
	public abstract double convert( double value, Unit newUnit ) throws CannotConvertException ;

	/**
	*  Returns the reference unit for this unit type.
	*
	*  @return  The reference unit for this type of unit object.
	**/
	public abstract Unit getRefUnit() ;

	/**
	*  Returns the factor for converting this unit object to the reference
	*  units for this type of unit.
	*
	*	@return  The conversion factor for converting this unit to the reference unit.
	**/
	protected abstract double factor() ;

	/**
	*  <p>  Converts a value with this unit to a value with the reference units
	*       for this unit.  This is more efficient than the following:  </p>
	*
	*  <p>  valueRU = <theUnits.convert>(value, <theUnits.getRefUnit>() );  </p>
	*
	*  <p>  Warning: The default version will not work with temperature units.
	*       A different version of this routine must be provided for temperature
	*       units.
	*  </p>
	*
	*  @param   value  The value in the "this" units to be converted
	*                  to "Reference Units".
	*  @return  The value converted to the reference units.
	**/
	public double convertToRef( double value ) {
		//	Reference unit factor always equals 1.0.
		return value / this.factor();
	}

	/**
	*  <p>  Converts a value with this unit's reference units to a value with
	*       this unit.
	*  </p>
	*
	*  <p>  Warning: The default version will not work with temperature units.
	*       A different version of this routine must be provided for temperature
	*       units.
	*  </p>
	*
	*  @param   value  The value in the "Reference Units" to be converted to "newUnit".
	*  @return  The value converted to the new units.
	**/
	public double convertFromRef( double value ) {
		//	Reference unit factor always equals 1.0.
		return value * this.factor();
	}

	/**
	*  Determines if two units are consistant with each other (ie: one can
	*  be converted into the other).
	*
	*  @param   oldUnit  The unit to be converted from.
	*  @param   newUnit  The unit to be converted to.
	*  @return  Returns true if the units are compatible, false if they are not.
	**/
	public static boolean compatible( Unit oldUnit, Unit newUnit ) {
	
		//	Get the object's superclass name
		String oldName = oldUnit.getClass().getSuperclass().getName();
		String newName = newUnit.getClass().getSuperclass().getName();
		
		//	Do the two Unit objects have different unit types (LengthUnit, TimeUnit, etc)?
		if ( ! oldName.equals( newName ) )
			return false;

		return true;
	}

	/**
	*  Returns the conversion factor required to convert one set
	*  of units to another.
	*
	*  @param   oldUnit  The unit to be converted from.
	*  @param   newUnit  The unit to be converted to.
	*  @return  The factor required to convert from oldUnit to newUnit.
	*  @exception  CannotConvertException  Unable to convert between the
	*              specified units.
	**/
	public static double cfactor( Unit oldUnit, Unit newUnit ) throws CannotConvertException {
		//	Do the two Unit objects have different unit types (LengthUnit, TimeUnit, etc)?
		if ( ! compatible( oldUnit, newUnit ) ) {
			throw new CannotConvertException(oldUnit.getClass().getSuperclass().getName()
			+ " to "
			+ newUnit.getClass().getSuperclass().getName() );
		}
		return (newUnit.factor() / oldUnit.factor());
	}

	/**
	*  Returns the conversion factor required to convert this unit
	*  to the reference units for this unit type.
	*
	*  @return  The factor required to convert from this unit to the reference
	*           units for this unit type.
	**/
	public double cfactor() {
		return this.factor();
	}
	
	/**
	*  Returns true if the conversion between this unit and this unit's
	*  reference unit is exact.  False is returned otherwise (if the conversion
	*  is an approximation).
	*
	*  @return  Returns true if this unit is an exact conversion from the base unit.
	**/
	public final boolean isExact() {
		return exact;
	}

	/**
	*  Returns the name of this unit object (the name of the unit that it is).
	*  Example:  If the object has the units of meters, then "meter" is returned.
	*
	*  @return  The name of this object's unit.
	**/
	public final String getName() {
		return name;
	}

	/**
	*  Returns the abreviation of this unit object.
	*  Example:  If the object has the units of meters, then "m" is returned.
	*
	*  @return  The abreviation of this object's unit name.
	**/
	public final String getAbreviation() {
		return abv;
	}

	/**
	*  Returns the abreviation of this unit object.
	*  Example:  If the object has the units of meters, then "m" is returned.
	*
	*  @return  The abreviation of this object's unit name.
	**/
	public String toString() {
		return getAbreviation();
	}

	/**
	*  The result is true if and only if the argument is not null
	*  and is a Unit object that has exactly the same unit type as
	*  this object.
	*
	*  @param   obj  The object to be compared with this Unit object for equality.
	*  @return  True if the argument is not null and is a Unit object that has exactly
	*           the same unit type as this object.
	**/
	public boolean equals( Object theUnit ) {
		if ( theUnit != null && theUnit instanceof Unit
				&& Unit.compatible( this, (Unit) theUnit ) ) {
			Class unitC = theUnit.getClass();
			Class currC = this.getClass();
			//	Return true if the two objects are of the same class name.
			return (unitC.getName().equals( currC.getName() ));
			
		} else
			return false;
	}

	/**
	*  Returns the hashcode for this Unit object.  This is a unique
	*  number identifying each and every unit type.
	*
	*  @return  Returns the hascode for this Unit object.
	**/
	public int hashCode() {
		return this.getClass().getName().hashCode();
	}

	/**
	*  Make a copy of this unit (since most Unit classes are singletons, this
	*  will by default just return a refrence to the one instance of this
	*  particular Unit class).
	*  WARNING:  If you create a Unit subclass that is not an immutable
	*  singleton, then you will need to override this function!
	*
	*	@return  Returns a clone of this unit.
	**/
	public Object clone() throws CloneNotSupportedException {
		//	Just return a reference to this instance of this class (since only one
		//	instance can exist at a time).
		return this;
	}


}


